@@ -1,6 +1,6 @@
 /* about.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2011 Ralf Hoffmann.
+ * Copyright (C) 2011-2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -34,7 +34,7 @@
 #include "datei.h"
 #include "worker_locale.h"
 
-About::About( AGUIX *aguix ) : m_aguix( aguix )
+About::About( AGUIX *aguix, Worker &worker ) : m_aguix( aguix ), m_worker( worker )
 {
 }
 
@@ -90,7 +90,7 @@ void About::about()
     ac1_texts->setMaxSpace( 5 );
     ac1_texts->setBorderWidth( 5 );
 
-    std::string str1 = AGUIXUtils::formatStringToString( "Worker %d.%d.%d%s|Copyright (C) 1998 - 2011 Ralf Hoffmann||Active options:",
+    std::string str1 = AGUIXUtils::formatStringToString( "Worker %d.%d.%d%s|Copyright (C) 1998 - 2012 Ralf Hoffmann||Active options:",
                                                          WORKER_MAJOR, WORKER_MINOR, WORKER_PATCH, WORKER_COMMENT );
     win->addMultiLineText( str1, *ac1_texts, 0, 0, NULL, NULL );
 
@@ -160,6 +160,14 @@ void About::about()
     option_lv->setPreColors( row, FieldListView::PRECOLOR_NOTSELORACT );
 #endif
 
+#ifdef HAVE_XFT
+    /* XFT */
+    row = option_lv->addRow();
+    option_lv->setText( row, 0, "XFT support:" );
+    option_lv->setText( row, 1, "yes" );
+    option_lv->setPreColors( row, FieldListView::PRECOLOR_NOTSELORACT );
+#endif
+
     /* libmagic */
     row = option_lv->addRow();
     option_lv->setText( row, 0, "File magic support:" );
@@ -180,6 +188,27 @@ void About::about()
 #endif
     option_lv->setPreColors( row, FieldListView::PRECOLOR_NOTSELORACT );
 
+    /* udisks */
+    row = option_lv->addRow();
+    option_lv->setText( row, 0, "UDisks support:" );
+#ifdef HAVE_DBUS
+    option_lv->setText( row, 1, "yes" );
+#else
+    option_lv->setText( row, 1, "no" );
+#endif
+    option_lv->setPreColors( row, FieldListView::PRECOLOR_NOTSELORACT );
+
+#ifdef HAVE_DBUS
+    row = option_lv->addRow();
+    option_lv->setText( row, 0, "  UDisks available:" );
+    if ( m_worker.getVolumeManager().getUDisksAvailable() == true ) {
+        option_lv->setText( row, 1, "yes" );
+    } else {
+        option_lv->setText( row, 1, "no" );
+    }
+    option_lv->setPreColors( row, FieldListView::PRECOLOR_NOTSELORACT );
+#endif
+
     /* EXE_STRING_LEN */
     row = option_lv->addRow();
     option_lv->setText( row, 0, "Maximum length of command lines:" );
