@@ -138,7 +138,6 @@ AGUIX::AGUIX( int argc, char **argv, const std::string &classname ) : m_argc( ar
   m_system_colors = 0;
   privatecmap=false;
   cutstart=NULL;
-  cutbuffer=NULL;
   pastestart = NULL;
   backpm = None;
   scr = -1;
@@ -286,7 +285,6 @@ AGUIX::~AGUIX()
     }
   }
   cancelCut();
-  if(cutbuffer!=NULL) _freesafe(cutbuffer);
   _freesafe( keybuf );
 }
 
@@ -798,6 +796,9 @@ Message *AGUIX::wait4mess( int mode )
 	if( ( LastEvent.xselectionclear.window==cutstart->getWindow() ) &&
 	    ( LastEvent.xselection.selection==XA_PRIMARY ) ) {
 	  cancelCut();
+        } else if( LastEvent.xselectionclear.window == getGroupWin() &&
+                   LastEvent.xselection.selection==XA_PRIMARY ) {
+            clipboard_string = "";
 	}
       }
       break;
@@ -850,8 +851,8 @@ Message *AGUIX::wait4mess( int mode )
 			ev.xselection.target,
 			8,
 			PropModeReplace,
-			(unsigned char*)cutbuffer,
-			strlen(cutbuffer));
+			(unsigned char*)clipboard_string.c_str(),
+			clipboard_string.length());
       } else {
 	ev.xselection.property=None;
       }
@@ -997,6 +998,13 @@ int AGUIX::msgHandler( int mode, AWindow *parent, bool onlyExpose )
       AGUIX_freeMessage( tmsg );
       lastTimerEventNr = timerEventNr;
     }
+
+    // test again for timer since the timer message could
+    // have disabled the timer
+    if ( ! ( ( msgLockElement != NULL ) &&
+	     ( timerEnabled == true ) ) ) {
+        break;
+    }
   }
   return XEventsQueued( dsp, QueuedAfterFlush );
 }
@@ -1458,8 +1466,7 @@ int AGUIX::startCut(GUIElement *elem,const char *buffer)
   XSetSelectionOwner(dsp,XA_PRIMARY,elem->getWindow(),CurrentTime);
   if(elem->getWindow()==XGetSelectionOwner(dsp,XA_PRIMARY)) {
     cutstart=elem;
-    if(cutbuffer!=NULL) _freesafe(cutbuffer);
-    cutbuffer=dupstring(buffer);
+    clipboard_string = buffer;
   } else {
     cancelCut();
     return 1;
@@ -1489,7 +1496,7 @@ bool AGUIX::amiOwner() const
 
 const char *AGUIX::getCutBuffer() const
 {
-  return cutbuffer;
+    return clipboard_string.c_str();
 }
 
 void AGUIX::requestCut(Window win)
@@ -2828,3 +2835,14 @@ void AGUIX::destroyBGHandlers()
     m_bg_handlers.clear();
     m_bg_handler_cleanup_list.clear();
 }
+
+void AGUIX::copyToClipboard( const std::string &s )
+{
+    clipboard_string = s;
+
+    XStoreBytes( dsp, s.c_str(), s.length() );
+    XSetSelectionOwner( dsp, XA_PRIMARY, getGroupWin(), CurrentTime );
+    if ( getGroupWin() != XGetSelectionOwner( dsp, XA_PRIMARY ) ) {
+        clipboard_string = "";
+    }
+}
