@@ -1,6 +1,6 @@
 /* aguix.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2010 Ralf Hoffmann.
+ * Copyright (C) 2001-2010,2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -236,6 +236,8 @@ public:
     void unregisterBGHandler( AWindow *window );
     void executeBGHandlers( AGMessage &msg );
     void destroyBGHandlers();
+
+    void copyToClipboard( const std::string &s );
 private:
   Display *dsp;
   int scr;
@@ -303,7 +305,6 @@ private:
 
   Cursor cursors[MAXCURSORS];
   GUIElement *cutstart,*pastestart;
-  char *cutbuffer;
 
   unsigned int rootWindowWidth, rootWindowHeight;
   Pixmap backpm;
@@ -344,6 +345,8 @@ private:
     /* temporary list for unregister to cleanup handler after execution */
     std::list< RefCount< BackgroundMessageHandler > > m_bg_handler_cleanup_list;
     std::list< AWindow *> m_bg_handler_erase_handler;
+
+    std::string clipboard_string;
 };
 
 #endif
