@@ -1,6 +1,6 @@
 /* fieldlistview.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2002-2010 Ralf Hoffmann.
+ * Copyright (C) 2002-2010,2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -2282,7 +2282,7 @@ void FieldListView::setMBG(int color)
   }
 }
 
-int FieldListView::maximizeX()
+int FieldListView::getMaximumWidth()
 {
   int s;
   int bw;
@@ -2290,11 +2290,19 @@ int FieldListView::maximizeX()
   bw = getBorderWidth();
   
   s = getMaxTextWidth() + ( ( vbar != 0 ) ? vbar_width : 0 ) + 2 * bw;
+  return s;
+}
+
+int FieldListView::maximizeX()
+{
+  int s;
+
+  s = getMaximumWidth();
   resize( s, getHeight() );
   return s;
 }
 
-int FieldListView::maximizeY()
+int FieldListView::getMaximumHeight()
 {
   int elem_height, s;
   int bw;
@@ -2306,6 +2314,14 @@ int FieldListView::maximizeY()
   s = elem_height * getElements() +
     ( ( hbar != 0 ) ? hbar_height : 0 ) + 2 * bw +
     ( ( showHeader == true ) ? getHeaderHeight() : 0 );
+  return s;
+}
+
+int FieldListView::maximizeY()
+{
+  int s;
+
+  s = getMaximumHeight();
   resize( getWidth(), s );
   return s;
 }
