@@ -681,7 +681,8 @@ int Requester::request( const char *title, TextStorage &ts, const char *buttons,
   ac1->add( tv, 0, 0, AContainer::CO_MIN );
   tv->create();
   tv->show();
-  tv->maximizeX();
+  tv->maximizeX( aguix->getRootWindowWidth() * 60 / 100 );
+  tv->setLineWrap( true );
   tv->maximizeYLines( 15 );
   ac1->readLimits();
 
