@@ -1,6 +1,6 @@
 /* textview.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2010 Ralf Hoffmann.
+ * Copyright (C) 2005-2011 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -189,84 +189,9 @@ bool TextView::handleMessage(XEvent *E,Message *msg)
     if ( ( getAcceptFocus() == true ) && ( getHasFocus() == true ) ) {
       if ( isVisible() == true ) {
 	if ( isTopParent( msg->window ) == true ) {
-	  // we have the focus so let's react to some keys
-	  // lets call an extra handler so it can be overwritten
-	  switch ( msg->key ) {
-	    case XK_Up:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( vbar->getOffset() - 1 );
-		redraw();
-	      }
-	      break;
-	    case XK_Left:
-	      if ( hbar != NULL ) {
-		hbar->setOffset( hbar->getOffset() - getHScrollStep() );
-		redraw();
-	      }
-	      break;
-	    case XK_Down:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( vbar->getOffset() + 1 );
-		redraw();
-	      }
-	      break;
-	    case XK_Right:
-	      if ( hbar != NULL ) {
-		hbar->setOffset( hbar->getOffset() + getHScrollStep() );
-		redraw();
-	      }
-	      break;
-	    case XK_Home:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( 0 );
-		redraw();
-	      }
-	      break;
-	    case XK_End:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( vbar->getMaxLen() );
-		redraw();
-	      }
-	      break;
-	    case XK_Prior:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( vbar->getOffset() - vbar->getMaxDisplay() + 1 );
-		redraw();
-	      }
-	      break;
-	    case XK_Next:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( vbar->getOffset() + vbar->getMaxDisplay() - 1 );
-		redraw();
-	      }
-	      break;
-	    case XK_w:
-              if ( KEYSTATEMASK( msg->keystate ) == 0 ) {
-                setLineWrap( ( getLineWrap() == true ) ? false : true );
-              }
-	      break;
-	    case XK_l:
-              if ( KEYSTATEMASK( msg->keystate ) == 0 ) {
-                setShowLineNumbers( ( getShowLineNumbers() == true ) ? false : true );
-              }
-	      break;
-              
-              // another pair of handy shortcuts
-	    case XK_BackSpace:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( vbar->getOffset() - vbar->getMaxDisplay() + 1 );
-		redraw();
-	      }
-              break;
-	    case XK_space:
-	      if ( vbar != NULL ) {
-		vbar->setOffset( vbar->getOffset() + vbar->getMaxDisplay() - 1 );
-		redraw();
-	      }
-              break;
-	    default:
-	      break;
-	  }
+            // we have the focus so let's react to some keys
+            // lets call an extra handler so it can be overwritten
+            handleKeyMessage( msg->key, msg->keystate );
 	}
       }
     }
@@ -294,6 +219,8 @@ bool TextView::handleMessage(XEvent *E,Message *msg)
 	    redraw();
 	}
       }
+
+      checkForEndReached();
       
       _lastwheel = t2;
     } else if ( msg->button == Button1 ) {
@@ -324,6 +251,7 @@ void TextView::gui_callback( Widget *elem, int value )
 {
   if ( ( elem == vbar ) || ( elem == hbar ) ) {
     redraw();
+    checkForEndReached();
   }
 }
 
@@ -477,10 +405,10 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
         
         // ask how much bytes fit into the invisible area (due to horizontal scrolling)
         int hidden_length, hidden_width;
-        hidden_length = _aguix->getStrlen4WidthMaxlen( s1.c_str(), hbar->getOffset() / 2, hbar->getOffset(), &hidden_width );
+        hidden_length = ts.getAWidth().getStrlen4WidthMaxlen( s1.c_str(), hbar->getOffset() / 2, hbar->getOffset(), &hidden_width );
         
         int overscan_width = width + ( hbar->getOffset() - hidden_width );
-        int visible_length = _aguix->getStrlen4WidthMaxlen( s1.c_str() + hidden_length, overscan_width / 2, overscan_width, NULL );
+        int visible_length = ts.getAWidth().getStrlen4WidthMaxlen( s1.c_str() + hidden_length, overscan_width / 2, overscan_width, NULL );
         
         std::pair<int,int> cur_line = ts.getRealLinePair( vbar->getOffset() + i );
 
@@ -517,9 +445,9 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
                                           start_x - hbar->getOffset() + hidden_width, i * elementHeight + start_y, fg );
                         
                         hidden_length += s2.length();
-                        hidden_width += _aguix->getTextWidth( s2.c_str() );
+                        hidden_width += ts.getAWidth().getWidth( s2.c_str() );
                         visible_length -= s2.length();
-                        overscan_width -= _aguix->getTextWidth( s2.c_str() );
+                        overscan_width -= ts.getAWidth().getWidth( s2.c_str() );
                     } else {
                         // invalid selection start offset
                         // just draw the whole visible string
@@ -529,9 +457,9 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
                                           start_x - hbar->getOffset() + hidden_width, i * elementHeight + start_y, fg );
                         
                         hidden_length += s2.length();
-                        hidden_width += _aguix->getTextWidth( s2.c_str() );
+                        hidden_width += ts.getAWidth().getWidth( s2.c_str() );
                         visible_length -= s2.length();
-                        overscan_width -= _aguix->getTextWidth( s2.c_str() );
+                        overscan_width -= ts.getAWidth().getWidth( s2.c_str() );
                     }
                 }
                 
@@ -552,7 +480,7 @@ void TextView::drawLines( int start_x, int start_y, int width, int lines )
                     
                     if ( selection_len > 0 ) {
                         std::string s2( s1, hidden_length, selection_len );
-                        int selection_width = _aguix->getTextWidth( s2.c_str() );
+                        int selection_width = ts.getAWidth().getWidth( s2.c_str() );
                         
                         _aguix->setFG( _colors.getSelectionBG() );
                         _aguix->FillRectangle( dc.getDrawable(),
@@ -621,7 +549,7 @@ void TextView::drawLineNumbers( int start_x, int end_x, int start_y, int lines )
     }
 }
 
-int TextView::setFont( char *fontname )
+int TextView::setFont( const char *fontname )
 {
   font = _aguix->getFont( fontname );
   last_w = last_h = last_cont_width = -1;
@@ -691,7 +619,7 @@ bool TextView::getDisplayFocus() const
   return _display_focus;
 }
 
-void TextView::maximizeX()
+void TextView::maximizeX( int max_width )
 {
   int l;
   int bw;
@@ -707,6 +635,10 @@ void TextView::maximizeX()
   if ( l >= _aguix->getRootWindowWidth() )
     l = _aguix->getRootWindowWidth();
 
+  if ( max_width > 0 && l > max_width ) {
+      l = max_width;
+  }
+
   resize( l, getHeight() );
 }
 
@@ -788,6 +720,8 @@ void TextView::jumpToLine( int line_nr, bool highlight_line )
   
   vbar->setOffset( l );
   redraw();
+
+  checkForEndReached();
 }
 
 TextView::ColorDef::ColorDef()
@@ -996,13 +930,13 @@ void TextView::makeSelectionVisible()
             updateHBarForVisibleLines();
 
             if ( ts.getLine( l, 0, actual_sel_start.second, str1 ) == 0 ) {
-                int selstart_offset = _aguix->getTextWidth( str1.c_str() );
+                int selstart_offset = ts.getAWidth().getWidth( str1.c_str() );
                 int selend_offset = -1;
                 int new_offset = hbar->getOffset();
                 
                 if ( linePairEQ( actual_sel_start.first, actual_sel_end.first ) == true ) {
                     if ( ts.getLine( l, 0, actual_sel_end.second, str1 ) == 0 ) {
-                        selend_offset = _aguix->getTextWidth( str1.c_str() );
+                        selend_offset = ts.getAWidth().getWidth( str1.c_str() );
                     }
                 }
 
@@ -1061,3 +995,107 @@ int TextView::getYOffset()
     }
     return -1;
 }
+
+bool TextView::handleKeyMessage( KeySym key,
+                                 unsigned int keystate )
+{
+    switch ( key ) {
+        case XK_Up:
+            if ( vbar != NULL ) {
+		setVBarOffset( vbar->getOffset() - 1 );
+		redraw();
+            }
+            break;
+        case XK_Left:
+            if ( hbar != NULL ) {
+		hbar->setOffset( hbar->getOffset() - getHScrollStep() );
+		redraw();
+            }
+            break;
+        case XK_Down:
+            if ( vbar != NULL ) {
+                setVBarOffset( vbar->getOffset() + 1 );
+		redraw();
+            }
+            break;
+        case XK_Right:
+            if ( hbar != NULL ) {
+		hbar->setOffset( hbar->getOffset() + getHScrollStep() );
+		redraw();
+            }
+            break;
+        case XK_Home:
+            if ( vbar != NULL ) {
+		setVBarOffset( 0 );
+		redraw();
+            }
+            break;
+        case XK_End:
+            if ( vbar != NULL ) {
+		setVBarOffset( vbar->getMaxLen() );
+		redraw();
+            }
+            break;
+        case XK_Prior:
+            if ( vbar != NULL ) {
+		setVBarOffset( vbar->getOffset() - vbar->getMaxDisplay() + 1 );
+		redraw();
+            }
+            break;
+        case XK_Next:
+            if ( vbar != NULL ) {
+		setVBarOffset( vbar->getOffset() + vbar->getMaxDisplay() - 1 );
+		redraw();
+            }
+            break;
+        case XK_w:
+            if ( KEYSTATEMASK( keystate ) == 0 ) {
+                setLineWrap( ( getLineWrap() == true ) ? false : true );
+            }
+            break;
+        case XK_l:
+            if ( KEYSTATEMASK( keystate ) == 0 ) {
+                setShowLineNumbers( ( getShowLineNumbers() == true ) ? false : true );
+            }
+            break;
+              
+            // another pair of handy shortcuts
+        case XK_BackSpace:
+            if ( vbar != NULL ) {
+		setVBarOffset( vbar->getOffset() - vbar->getMaxDisplay() + 1 );
+		redraw();
+            }
+            break;
+        case XK_space:
+            if ( vbar != NULL ) {
+		setVBarOffset( vbar->getOffset() + vbar->getMaxDisplay() - 1 );
+		redraw();
+            }
+            break;
+        default:
+            break;
+    }
+    return false;
+}
+
+void TextView::setVBarOffset( int offset )
+{
+    if ( vbar != NULL ) {
+        vbar->setOffset( offset );
+
+        checkForEndReached();
+    }
+}
+
+void TextView::checkForEndReached()
+{
+    if ( vbar != NULL ) {
+        if ( vbar->getOffset() + vbar->getMaxDisplay() >= vbar->getMaxLen() ) {
+	    AGMessage *agmsg = AGUIX_allocAGMessage();
+	    agmsg->type = AG_TEXTVIEW_END_REACHED;
+	    agmsg->textview.textview = this;
+
+            _aguix->putAGMsg( agmsg );
+        }
+    }
+}
