@@ -1,6 +1,6 @@
 /* configtokens.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2003-2011 Ralf Hoffmann.
+ * Copyright (C) 2003-2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -386,7 +386,11 @@ struct mytoken tokenlist[]={
   { SWITCHTOPREVBANK_WCP, "switchtoprevbank" },
   { SWITCHTOBANKNR_WCP, "switchtobanknr" },
   { BANKNR_WCP, "banknr" },
-  { USE_VIRTUAL_TEMP_COPIES_WCP, "usevirtualtempcopies" }
+  { USE_VIRTUAL_TEMP_COPIES_WCP, "usevirtualtempcopies" },
+  { PATHJUMPOP_WCP, "pathjumpop" },
+  { PATHJUMPALLOWDIRS_WCP, "pathjumpallowdirs" },
+  { CLIPBOARDOP_WCP, "clipboardop" },
+  { CLIPBOARDSTRING_WCP, "clipboardstring" }
 };
 
 ArrayList *tokenlist_sorted = NULL;
