@@ -64,6 +64,8 @@ CopyOpWin::CopyOpWin( AGUIX*taguix, bool _move )
   mintime = 1000000;
   lastbytes = 0;
   lastrate = 0.0;
+  lastbytes_global = 0;
+  lastrate_global = 0.0;
   
   glob_percent = 0;
   
@@ -190,6 +192,7 @@ void
 CopyOpWin::starttimer()
 {
   gettimeofday( &starttime, NULL );
+  lasttime_global = starttime;
 }
 
 void
@@ -297,7 +300,7 @@ CopyOpWin::redraw()
    *  1 when clicked Cancel-Button or closed window
    *  other not implemented */
   int returnvalue=0;
-  long rtime;
+  long rtime, rtime_at_current_rate;
   double drtime;
   struct timeval curtime;
   double difftime2, lastdiff2;
@@ -432,6 +435,14 @@ CopyOpWin::redraw()
       rate = (double)copied_bytes * 1000000.0 / difftime2;
     } else rate=0.0;
 
+    double lastdiff_global = diffgtod( &curtime, &lasttime_global );
+
+    if ( lastdiff_global >= 10.0 * mintime ) {
+      lastrate_global = (double)( copied_bytes - lastbytes_global ) * 1000000.0 / lastdiff_global;
+      lastbytes_global = copied_bytes;
+      lasttime_global = curtime;
+    }
+
     if ( HUMAN_READABLE_DIALOG ) {
         copied_bytes_str = AGUIXUtils::bytes_to_human_readable_f( (double)copied_bytes,
                                                                   1, ( rate > 0.0 ) ? 5 : 0,
@@ -514,13 +525,34 @@ CopyOpWin::redraw()
         rtime = (long)( drtime / 1000000.0 );
       } else
         rtime = 700000;  // no rate available => no real time calculatable
+
+      if ( lastrate_global != 0.0 ) {
+          double remaining_bytes = (double)bytes - (double)copied_bytes;
+          rtime_at_current_rate = remaining_bytes / lastrate_global;
+          rtime_at_current_rate++;
+      } else {
+          rtime_at_current_rate = 700000;
+      }
+
       rtime++; // add a second because the last second will shown as 0:00
                // and at least I don't like this
       if ( rtime > 604800 ) // more then 7 days
         sprintf( tstr, "--:--" );
       else 
         sprintf( tstr, "%ld:%02ld", rtime / 60, rtime % 60 );
-      timetext->setText(tstr);
+
+      if ( rtime_at_current_rate <= 604800 &&
+           fabs( rtime - rtime_at_current_rate ) > 10.0 &&
+           fabs( rtime - rtime_at_current_rate ) / (double)rtime > 0.1 ) {
+          std::string alt_time_str = AGUIXUtils::formatStringToString( catalog.getLocale( 973 ),
+                                                                       tstr,
+                                                                       rtime_at_current_rate / 60,
+                                                                       rtime_at_current_rate % 60 );
+          timetext->setText( alt_time_str.c_str() );
+      } else {
+          timetext->setText(tstr);
+      }
+
       tw = timetext->getX() + timetext->getWidth() + window->getBorderWidth();
       if ( tw > melw ) {
         melw = tw;
@@ -610,6 +642,8 @@ CopyOpWin::redraw()
     std::string str1;
     int erg;
     
+    stoptimer();
+
     // user canceled -> request to be sure
     str1 = catalog.getLocale( 11 );
     str1 += "|";
@@ -618,6 +652,8 @@ CopyOpWin::redraw()
                            catalog.getLocale( 543 ),
                            str1.c_str() );
     if ( erg != 0 ) returnvalue = 0;
+
+    conttimer();
   }
   
   return returnvalue;
