@@ -353,15 +353,13 @@ char *ParentDir(const char *pathstr,char *dirnamereturn)
 char *HandlePath(const char *pathstr)
 {
   char *newpath, *newpath2, *buffer;
-  int x,y,parentlevel;
+  int x,y;
   int start,end;
-  int found,ende;
+  int ende;
   
   buffer = (char*)_allocsafe( strlen( pathstr ) + 1 );
   x=strlen(pathstr);
   y=0;
-  found=0;
-  parentlevel=0;
   ende=0;
   newpath=(char*)_allocsafe(1);
   newpath[0]=0;
@@ -1299,7 +1297,8 @@ int Datei::getCharacter()
   return (int)( cf[0] );
 }
 
-int Datei::copyfile( const char *dest, const char *source )
+int Datei::copyfile( const char *dest, const char *source,
+                     CopyfileProgressCallback *progress_callback )
 {
   int fdw,fdr;
   bool useregcopy, cancel, follow_symlinks, isCorrupt;
@@ -1308,6 +1307,7 @@ int Datei::copyfile( const char *dest, const char *source )
   char *buf[BUFSIZE];
   worker_struct_stat st, dst;
   int erg;
+  loff_t total_bytes = 0, bytes_copied = 0;
 
   if ( ( dest == NULL ) || ( source == NULL ) ) return 1;
   
@@ -1323,11 +1323,16 @@ int Datei::copyfile( const char *dest, const char *source )
   if ( worker_stat( source, &dst ) != 0 ) return 1;
   
   useregcopy = false;
-  if ( S_ISREG( st.st_mode ) ) useregcopy = true;
-  else if ( ( S_ISLNK( st.st_mode ) ) &&
+  if ( S_ISREG( st.st_mode ) ) {
+      useregcopy = true;
+      total_bytes = st.st_size;
+  } else if ( ( S_ISLNK( st.st_mode ) ) &&
 	    ( follow_symlinks == true ) &&
 	    ( isCorrupt == false ) &&
-	    ( S_ISREG( dst.st_mode ) ) ) useregcopy = true;
+              ( S_ISREG( dst.st_mode ) ) ) {
+      useregcopy = true;
+      total_bytes = dst.st_size;
+  }
   
   if ( useregcopy == false ) return 1;
   
@@ -1360,6 +1365,13 @@ int Datei::copyfile( const char *dest, const char *source )
 	erg = 1;
 	break;
       }
+
+      bytes_copied += writebytes;
+
+      if ( progress_callback != NULL ) {
+          progress_callback->progress_callback( bytes_copied,
+                                                total_bytes );
+      }
     }
   } while ( ( readbytes > 0 ) &&
 	    ( cancel == false ) );
@@ -1564,3 +1576,7 @@ size_t Datei::read( char *buffer, size_t len )
     }
     return count;
 }
+
+CopyfileProgressCallback::~CopyfileProgressCallback()
+{
+}
