@@ -504,7 +504,6 @@ BookmarkDBEntry DirBookmarkUI::getActiveEntry( const std::string &cat )
 std::string DirBookmarkUI::switchCat( const std::string &current_cat, int dir )
 {
     std::list<std::string> cats = m_data.getCats();
-    bool update = false;
     std::string new_cat = current_cat;
 
     if ( dir == 0 || cats.empty() == true ) return "";
@@ -513,10 +512,8 @@ std::string DirBookmarkUI::switchCat( const std::string &current_cat, int dir )
         // "all"
         if ( dir > 0 ) {
             new_cat = cats.front();
-            update = true;
         } else {
             new_cat = cats.back();
-            update = true;
         }
     } else {
         std::list<std::string>::iterator it1 = std::find( cats.begin(),
@@ -532,18 +529,15 @@ std::string DirBookmarkUI::switchCat( const std::string &current_cat, int dir )
                     it1--;
                     new_cat = *it1;;
                 }
-                update = true;
             } else {
                 it1++;
                 if ( it1 == cats.end() )
                     new_cat = "";
                 else
                     new_cat = *it1;
-                update = true;
             }
         } else {
             new_cat = "";
-            update = true;
         }
     }
     return new_cat;
