@@ -1,6 +1,6 @@
 /* filereq.cc
- * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2007 Ralf Hoffmann.
+ * This file belongs to Worker, a file manager for UN*X/X11.
+ * Copyright (C) 2001-2007,2011 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -30,6 +30,10 @@
 #include <aguix/fieldlistview.h>
 #include <aguix/stringgadget.h>
 #include <aguix/request.h>
+#include "verzeichnis.hh"
+#include "fileentry.hh"
+#include "simplelist.hh"
+#include "datei.h"
 
 FileRequester::FileRequester(AGUIX *parent)
 {
