@@ -1,6 +1,6 @@
 /* flagreplacer.cc
- * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2006 Ralf Hoffmann.
+ * This file belongs to Worker, a file manager for UN*X/X11.
+ * Copyright (C) 2006,2011 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,12 @@
 
 #include "flagreplacer.hh"
 #include <aguix/lowlevelfunc.h>
+#include <aguix/awindow.h>
+#include <aguix/acontainer.h>
+#include <aguix/button.h>
+#include <aguix/fieldlistview.h>
 #include "worker.h"
+#include "worker_locale.h"
 
 FlagReplacer::FlagReplacer( FlagProducer &flagstore ) : _flagstore( flagstore )
 {
