@@ -1,6 +1,6 @@
 /* goftpop.cc
- * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2005-2007 Ralf Hoffmann.
+ * This file belongs to Worker, a file manager for UN*X/X11.
+ * Copyright (C) 2005-2007,2010-2011 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -24,6 +24,12 @@
 #include "normalmode.h"
 #include "worker.h"
 #include "avfssupport.hh"
+#include "worker_locale.h"
+#include "datei.h"
+#include <aguix/stringgadget.h>
+#include <aguix/choosebutton.h>
+#include <aguix/cyclebutton.h>
+#include <aguix/button.h>
 
 const char *GoFTPOp::name = "GoFTPOp";
 
@@ -155,7 +161,7 @@ GoFTPOp::save(Datei *fh)
   fh->configPutPairString( "username", user.c_str() );
   fh->configPutPairString( "password", pass.c_str() );
   fh->configPutPairBool( "alwaysstorepw", always_store_pw );
-  //TODO store avfs_module
+  fh->configPutPairString( "avfsmodule", avfs_module.c_str() );
   return true;
 }
 
@@ -249,15 +255,19 @@ GoFTPOp::doconfigure(int mode)
       ac1_6->setMinSpace( 5 );
       ac1_6->setMaxSpace( 5 );
       ac1_6->setBorderWidth( 0 );
-      //TODO translate
-      ac1_6->add( new Text( aguix, 0, 0, "AVFS module:"/*catalog.getLocale( 621 )*/, 1 ), 0, 0, cfix );
+      ac1_6->add( new Text( aguix, 0, 0, catalog.getLocale( 933 ), 1 ), 0, 0, cfix );
       cycb_avfs = (CycleButton*)ac1_6->add( new CycleButton( aguix, 0, 0, 50, 1, 0, 0 ),
                                                          1, 0, AContainer::CO_INCWNR );
 
-      for ( std::vector<std::string>::iterator it1 = module_list.begin();
+      int pos = 0;
+      for ( std::vector<std::string>::const_iterator it1 = module_list.begin();
             it1 != module_list.end();
-            it1++ ) {
+            it1++, pos++ ) {
           cycb_avfs->addOption( it1->c_str() );
+
+          if ( it1->c_str() == avfs_module ) {
+              cycb_avfs->setOption( pos );
+          }
       }
       cycb_avfs->resize( cycb_avfs->getMaxSize(), cycb_avfs->getHeight() );
       ac1_6->readLimits();
