@@ -1,6 +1,6 @@
 /* hw_volume_manager.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009-2010 Ralf Hoffmann.
+ * Copyright (C) 2009-2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -47,7 +47,10 @@ HWVolumeManager::HWVolumeManager( const std::string &fstab_location,
 #ifdef HAVE_HAL_DBUS
     m_dbus_conn( NULL ),
     m_hal_ctx( NULL ),
+#elif defined(HAVE_DBUS)
+    m_dbus_conn( NULL ),
 #endif
+    m_udisks_available( false ),
     m_fstab_table( fstab_location ),
     m_mtab_table( mtab_location ),
     m_mount_command( HW_VM_MOUNT_CMD ),
@@ -167,9 +170,104 @@ std::list<HWVolume> HWVolumeManager::getVolumes()
             }
         }
     }
+#elif defined(HAVE_DBUS)
+    if ( HALreconnect() == 0 && m_udisks_available)  {
+        DBusMessage *message, *reply;
+        DBusMessageIter reply_iter, sub_iter;
+        DBusError dbus_error;
+
+        message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                                "/org/freedesktop/UDisks",
+                                                "org.freedesktop.UDisks",
+                                                "EnumerateDevices" );
+
+        if ( message != NULL ) {
+
+            dbus_error_init( &dbus_error );
+
+            reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                               message,
+                                                               -1,
+                                                               &dbus_error );
+
+            dbus_message_unref( message );
+
+            if ( dbus_error_is_set( &dbus_error ) ) {
+                fprintf( stderr, "Couldn't enumerate udisks devices: %s\n", dbus_error.message );
+                dbus_error_free( &dbus_error );
+            } else if ( reply == NULL ) {
+                fprintf( stderr, "Couldn't enumerate udisks devices: %s\n", dbus_error.message );
+            } else {
+
+                dbus_message_iter_init( reply, &reply_iter );
+
+                if ( dbus_message_iter_get_arg_type( &reply_iter ) ==
+                     DBUS_TYPE_ARRAY ) {
+
+                    dbus_message_iter_recurse( &reply_iter,
+                                               &sub_iter );
+
+                    while ( dbus_message_iter_get_arg_type( &sub_iter ) ==
+                            DBUS_TYPE_OBJECT_PATH ) {
+                        const char *udi = NULL;
+
+                        dbus_message_iter_get_basic( &sub_iter, &udi );
+
+                        if ( udi ) {
+                            bool use_volume = false;
+
+                            // first check whether we should ignore the volume
+                            try {
+                                if ( ! getBoolProperty( udi, "DeviceIsSystemInternal" ) ) {
+                                    use_volume = true;
+                                }
+                            } catch ( int ) {
+                            }
+
+                            // now check whether the volume is actually a mountable filesystem
+                            if ( use_volume ) {
+                                try {
+                                    if ( getStringProperty( udi, "IdUsage" ) != "filesystem" ) {
+                                        use_volume = false;
+                                    }
+                                } catch ( int ) {
+                                }
+
+                                // no filesystem but ejectable?
+                                if ( ! use_volume ) {
+                                    try {
+                                        if ( getBoolProperty( udi, "DriveIsMediaEjectable" ) ) {
+                                            use_volume = true;
+                                        }
+                                    } catch ( int ) {
+                                    }
+                                }
+                            }
+
+                            if ( use_volume ) {
+                                HWVolume v( udi );
+
+                                update( v );
+
+                                if ( managed.count( getOSDevice( v.getDevice() ) ) < 1 ) {
+                                    managed.insert( getOSDevice( v.getDevice() ) );
+                
+                                    l.push_back( v );
+                                }
+                            }
+                        }
+
+                        dbus_message_iter_next( &sub_iter );
+                    }
+                }
+
+                dbus_message_unref( reply );
+            }
+        }
+    }
 #endif
 
-#ifndef HAVE_HAL_DBUS
+#if !defined(HAVE_HAL_DBUS) and !defined(HAVE_DBUS)
     for ( std::list< std::string >::iterator it1 = m_avail_parts.begin();
           it1 != m_avail_parts.end();
           it1++ ) {
@@ -193,7 +291,7 @@ int HWVolumeManager::mount( HWVolume &vol, std::string &error_return )
     if ( vol.isMounted() == true ) return 0;
 
     if ( ! vol.getUDI().empty() && vol.isInFStab() == false ) {
-        return halMount( vol, error_return );
+        return dbusMount( vol, error_return );
     } else {
         return cmdMount( vol, error_return );
     }
@@ -204,7 +302,7 @@ int HWVolumeManager::umount( HWVolume &vol, std::string &error_return )
     if ( vol.isMounted() == false ) return 0;
 
     if ( ! vol.getUDI().empty() && vol.isInFStab() == false ) {
-        return halUnmount( vol, error_return );
+        return dbusUnmount( vol, error_return );
     } else {
         return cmdUnmount( vol, error_return );
     }
@@ -213,7 +311,7 @@ int HWVolumeManager::umount( HWVolume &vol, std::string &error_return )
 int HWVolumeManager::eject( HWVolume &vol, std::string &error_return )
 {
     if ( ! vol.getUDI().empty() ) {
-        return halEject( vol, error_return );
+        return dbusEject( vol, error_return );
     } else {
         return cmdEject( vol, error_return );
     }
@@ -222,7 +320,7 @@ int HWVolumeManager::eject( HWVolume &vol, std::string &error_return )
 int HWVolumeManager::closeTray( HWVolume &vol, std::string &error_return )
 {
     if ( ! vol.getUDI().empty() ) {
-        return halCloseTray( vol, error_return );
+        return dbusCloseTray( vol, error_return );
     } else {
         return cmdCloseTray( vol, error_return );
     }
@@ -234,8 +332,13 @@ void HWVolumeManager::update( HWVolume &vol )
         bool mounted = false;
         
         try {
+#ifdef HAVE_HAL_DBUS
             mounted = getBoolProperty( vol.getUDI(),
                                        "volume.is_mounted" );
+#elif defined(HAVE_DBUS)
+            mounted = getBoolProperty( vol.getUDI(),
+                                       "DeviceIsMounted" );
+#endif
         } catch ( int ) {
         }
 
@@ -245,8 +348,16 @@ void HWVolumeManager::update( HWVolume &vol )
             std::string mountpoint;
 
             try  {
+#ifdef HAVE_HAL_DBUS
                 mountpoint = getStringProperty( vol.getUDI().c_str(),
                                                 "volume.mount_point" );
+#elif defined(HAVE_DBUS)
+                std::list< std::string > mountpoints = getStrlistProperty( vol.getUDI().c_str(),
+                                                                           "DeviceMountPaths" );
+                if ( ! mountpoints.empty() ) {
+                    mountpoint = *mountpoints.begin();
+                }
+#endif
             } catch ( int ) {
             }
 
@@ -261,7 +372,12 @@ void HWVolumeManager::update( HWVolume &vol )
         try {
             std::string label;
             
+#ifdef HAVE_HAL_DBUS
             label = getStringProperty( vol.getUDI().c_str(), "volume.label" );
+#elif defined(HAVE_DBUS)
+            label = getStringProperty( vol.getUDI().c_str(),
+                                       "IdLabel" );
+#endif
             
             if ( ! label.empty() ) {
                 vol.setLabel( label );
@@ -272,7 +388,11 @@ void HWVolumeManager::update( HWVolume &vol )
         try {
             loff_t size;
             
+#ifdef HAVE_HAL_DBUS
             size = getUint64Property( vol.getUDI().c_str(), "volume.size" );
+#elif defined(HAVE_DBUS)
+            size = getUint64Property( vol.getUDI().c_str(), "DeviceSize" );
+#endif
             
             if ( size != 0 ) {
                 vol.setSize( size );
@@ -282,8 +402,13 @@ void HWVolumeManager::update( HWVolume &vol )
 
         std::string device;
         try {
+#ifdef HAVE_HAL_DBUS
             device = getStringProperty( vol.getUDI().c_str(),
                                         "block.device" );
+#elif defined(HAVE_DBUS)
+            device = getStringProperty( vol.getUDI().c_str(),
+                                        "DeviceFile" );
+#endif
         } catch ( int ) {
         }
 
@@ -479,6 +604,78 @@ bool HWVolumeManager::getBoolProperty( const std::string &udi,
 
     return libhal_device_get_property_bool( m_hal_ctx, udi.c_str(),
                                             property.c_str(), NULL );
+#elif defined(HAVE_DBUS)
+    DBusMessage *message, *reply;
+    DBusMessageIter iter, reply_iter, sub_iter;
+    DBusError dbus_error;
+
+    message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                            udi.c_str(),
+                                            "org.freedesktop.DBus.Properties",
+                                            "Get" );
+
+    if ( message == NULL ) {
+        throw 1;
+    }
+
+    dbus_message_iter_init_append( message, &iter );
+
+    const char *arg1 = "org.freedesktop.UDisks.Device";
+    const char *arg2 = property.c_str();
+
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg1 );
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg2 );
+
+    dbus_error_init( &dbus_error );
+
+    reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                       message,
+                                                       -1,
+                                                       &dbus_error );
+
+    dbus_message_unref( message );
+
+    if ( dbus_error_is_set( &dbus_error ) ) {
+        fprintf( stderr, "get bool property error: %s\n", dbus_error.message );
+        dbus_error_free( &dbus_error );
+
+        throw 2;
+    }
+
+    if ( reply == NULL ) {
+        throw 3;
+    }
+
+    dbus_message_iter_init( reply, &reply_iter );
+
+    dbus_bool_t v = false;
+
+    if ( dbus_message_iter_get_arg_type( &reply_iter ) ==
+         DBUS_TYPE_VARIANT ) {
+
+        dbus_message_iter_recurse( &reply_iter,
+                                   &sub_iter );
+
+        if ( dbus_message_iter_get_arg_type( &sub_iter ) ==
+             DBUS_TYPE_BOOLEAN ) {
+
+            dbus_message_iter_get_basic( &sub_iter, &v );
+        } else {
+            dbus_message_unref( reply );
+            throw 4;
+        }
+    } else {
+        dbus_message_unref( reply );
+        throw 4;
+    }
+
+    dbus_message_unref( reply );
+
+    return v;
 #else
     throw -1;
 #endif
@@ -503,6 +700,84 @@ std::string HWVolumeManager::getStringProperty( const std::string &udi,
     std::string s2( s1 );
     libhal_free_string( s1 );
     return s2;
+#elif defined(HAVE_DBUS)
+    DBusMessage *message, *reply;
+    DBusMessageIter iter, reply_iter, sub_iter;
+    DBusError dbus_error;
+
+    message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                            udi.c_str(),
+                                            "org.freedesktop.DBus.Properties",
+                                            "Get" );
+
+    if ( message == NULL ) {
+        throw 1;
+    }
+
+    dbus_message_iter_init_append( message, &iter );
+
+    const char *arg1 = "org.freedesktop.UDisks.Device";
+    const char *arg2 = property.c_str();
+
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg1 );
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg2 );
+
+    dbus_error_init( &dbus_error );
+
+    reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                       message,
+                                                       -1,
+                                                       &dbus_error );
+
+    dbus_message_unref( message );
+
+    if ( dbus_error_is_set( &dbus_error ) ) {
+        fprintf( stderr, "get bool property error: %s\n", dbus_error.message );
+        dbus_error_free( &dbus_error );
+
+        throw 2;
+    }
+
+    if ( reply == NULL ) {
+        throw 3;
+    }
+
+    dbus_message_iter_init( reply, &reply_iter );
+
+    std::string res1;
+
+    if ( dbus_message_iter_get_arg_type( &reply_iter ) ==
+         DBUS_TYPE_VARIANT ) {
+
+        dbus_message_iter_recurse( &reply_iter,
+                                   &sub_iter );
+
+        if ( dbus_message_iter_get_arg_type( &sub_iter ) ==
+             DBUS_TYPE_STRING ) {
+
+            const char *str1 = NULL;
+
+            dbus_message_iter_get_basic( &sub_iter, &str1 );
+
+            if ( str1 ) {
+                res1 = str1;
+            }
+        } else {
+            dbus_message_unref( reply );
+            throw 4;
+        }
+    } else {
+        dbus_message_unref( reply );
+        throw 4;
+    }
+
+    dbus_message_unref( reply );
+
+    return res1;
 #else
     throw -1;
 #endif
@@ -525,6 +800,82 @@ loff_t HWVolumeManager::getUint64Property( const std::string &udi,
                                               property.c_str(), NULL );
     //TODO some error handling would be good
     return (loff_t)val1;
+#elif defined(HAVE_DBUS)
+    DBusMessage *message, *reply;
+    DBusMessageIter iter, reply_iter, sub_iter;
+    DBusError dbus_error;
+
+    message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                            udi.c_str(),
+                                            "org.freedesktop.DBus.Properties",
+                                            "Get" );
+
+    if ( message == NULL ) {
+        throw 1;
+    }
+
+    dbus_message_iter_init_append( message, &iter );
+
+    const char *arg1 = "org.freedesktop.UDisks.Device";
+    const char *arg2 = property.c_str();
+
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg1 );
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg2 );
+
+    dbus_error_init( &dbus_error );
+
+    reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                       message,
+                                                       -1,
+                                                       &dbus_error );
+
+    dbus_message_unref( message );
+
+    if ( dbus_error_is_set( &dbus_error ) ) {
+        fprintf( stderr, "get bool property error: %s\n", dbus_error.message );
+        dbus_error_free( &dbus_error );
+
+        throw 2;
+    }
+
+    if ( reply == NULL ) {
+        throw 3;
+    }
+
+    dbus_message_iter_init( reply, &reply_iter );
+
+    loff_t res1 = 0;
+
+    if ( dbus_message_iter_get_arg_type( &reply_iter ) ==
+         DBUS_TYPE_VARIANT ) {
+
+        dbus_message_iter_recurse( &reply_iter,
+                                   &sub_iter );
+
+        if ( dbus_message_iter_get_arg_type( &sub_iter ) ==
+             DBUS_TYPE_UINT64 ) {
+
+            dbus_uint64_t v;
+
+            dbus_message_iter_get_basic( &sub_iter, &v );
+
+            res1 = v;
+        } else {
+            dbus_message_unref( reply );
+            throw 4;
+        }
+    } else {
+        dbus_message_unref( reply );
+        throw 4;
+    }
+
+    dbus_message_unref( reply );
+
+    return res1;
 #else
     throw -1;
 #endif
@@ -554,6 +905,93 @@ std::list< std::string > HWVolumeManager::getStrlistProperty( const std::string
     libhal_free_string_array( l1 );
 
     return l2;
+#elif defined(HAVE_DBUS)
+    DBusMessage *message, *reply;
+    DBusMessageIter iter, reply_iter, sub_iter, sub_sub_iter;
+    DBusError dbus_error;
+
+    message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                            udi.c_str(),
+                                            "org.freedesktop.DBus.Properties",
+                                            "Get" );
+
+    if ( message == NULL ) {
+        throw 1;
+    }
+
+    dbus_message_iter_init_append( message, &iter );
+
+    const char *arg1 = "org.freedesktop.UDisks.Device";
+    const char *arg2 = property.c_str();
+
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg1 );
+    dbus_message_iter_append_basic( &iter,
+                                    DBUS_TYPE_STRING,
+                                    &arg2 );
+
+    dbus_error_init( &dbus_error );
+
+    reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                       message,
+                                                       -1,
+                                                       &dbus_error );
+
+    dbus_message_unref( message );
+
+    if ( dbus_error_is_set( &dbus_error ) ) {
+        fprintf( stderr, "get bool property error: %s\n", dbus_error.message );
+        dbus_error_free( &dbus_error );
+
+        throw 2;
+    }
+
+    if ( reply == NULL ) {
+        throw 3;
+    }
+
+    dbus_message_iter_init( reply, &reply_iter );
+
+    std::list< std::string > res;
+
+    if ( dbus_message_iter_get_arg_type( &reply_iter ) ==
+         DBUS_TYPE_VARIANT ) {
+
+        dbus_message_iter_recurse( &reply_iter,
+                                   &sub_iter );
+
+
+        if ( dbus_message_iter_get_arg_type( &sub_iter ) ==
+             DBUS_TYPE_ARRAY ) {
+
+            dbus_message_iter_recurse( &sub_iter,
+                                       &sub_sub_iter );
+
+            while ( dbus_message_iter_get_arg_type( &sub_sub_iter ) ==
+                    DBUS_TYPE_STRING ) {
+                const char *str1 = NULL;
+
+                dbus_message_iter_get_basic( &sub_sub_iter, &str1 );
+
+                if ( str1 ) {
+                    res.push_back( str1 );
+                }
+
+                dbus_message_iter_next( &sub_sub_iter );
+            }
+        } else {
+            dbus_message_unref( reply );
+            throw 4;
+        }
+    } else {
+        dbus_message_unref( reply );
+        throw 4;
+    }
+
+    dbus_message_unref( reply );
+
+    return res;
 #else
     throw -1;
 #endif
@@ -595,7 +1033,7 @@ void HWVolumeManager::setCloseTrayCommand( const std::string &cmd )
     }
 }
 
-int HWVolumeManager::halMount( HWVolume &vol, std::string &error_return )
+int HWVolumeManager::dbusMount( HWVolume &vol, std::string &error_return )
 {
     if ( vol.isMounted() == true ) {
         error_return = catalog.getLocale( 889 );
@@ -722,6 +1160,57 @@ int HWVolumeManager::halMount( HWVolume &vol, std::string &error_return )
             error_return = "Mount failed: No Mount method";
         }
     }
+#elif defined(HAVE_DBUS)
+    if ( HALreconnect() == 0 ) {
+        DBusMessage *message, *reply;
+        DBusError dbus_error;
+        const char *fs = "";
+        const char *options[] = { NULL };
+        int num_options = 0;
+
+        message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                                vol.getUDI().c_str(),
+                                                "org.freedesktop.UDisks.Device",
+                                                "FilesystemMount" );
+
+        if ( message == NULL ) {
+            error_return = "Mount failed: Unable to create DBus message";
+        } else {
+
+            const char **options_p = options;
+            
+            if ( ! dbus_message_append_args( message, 
+                                             DBUS_TYPE_STRING, &fs,
+                                             DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &options_p, num_options,
+                                             DBUS_TYPE_INVALID ) ) {
+                //TODO translate?
+                error_return = "Mount failed: Unable to add arguments to DBus message";
+                dbus_message_unref( message );
+            } else {
+
+                dbus_error_init( &dbus_error );
+
+                reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                                   message,
+                                                                   -1,
+                                                                   &dbus_error );
+
+                dbus_message_unref( message );
+
+                if ( dbus_error_is_set( &dbus_error ) ) {
+                    error_return = AGUIXUtils::formatStringToString( catalog.getLocale( 905 ),
+                                                                     dbus_error.message );
+                    dbus_error_free( &dbus_error );
+                } else {
+                    mount_failed = false;
+
+                    if ( reply ) {
+                        dbus_message_unref( reply );
+                    }
+                }
+            }
+        }
+    }
 #endif
     if ( mount_failed ) return 1;
 
@@ -772,7 +1261,7 @@ int HWVolumeManager::cmdMount( HWVolume &vol, std::string &error_return )
     return 0;
 }
 
-int HWVolumeManager::halUnmount( HWVolume &vol, std::string &error_return )
+int HWVolumeManager::dbusUnmount( HWVolume &vol, std::string &error_return )
 {
     if ( vol.isMounted() == false ) {
         error_return = catalog.getLocale( 893 );
@@ -845,7 +1334,7 @@ int HWVolumeManager::halUnmount( HWVolume &vol, std::string &error_return )
 #if 0
                 int hal_retcode;
 
-                // see comment in halMount
+                // see comment in dbusMount
                 if ( ! dbus_message_get_args( reply, 
                                               &error,
                                               DBUS_TYPE_UINT32, &hal_retcode,
@@ -875,6 +1364,55 @@ int HWVolumeManager::halUnmount( HWVolume &vol, std::string &error_return )
             error_return = "Unmount failed: No Unmount method";
         }
     }
+#elif defined(HAVE_DBUS)
+    if ( HALreconnect() == 0 ) {
+        DBusMessage *message, *reply;
+        DBusError dbus_error;
+        const char *options[] = { NULL };
+        int num_options = 0;
+
+        message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                                vol.getUDI().c_str(),
+                                                "org.freedesktop.UDisks.Device",
+                                                "FilesystemUnmount" );
+
+        if ( message == NULL ) {
+            error_return = "Unmount failed: Unable to create DBus message";
+        } else {
+
+            const char **options_p = options;
+            
+            if ( ! dbus_message_append_args( message, 
+                                             DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &options_p, num_options,
+                                             DBUS_TYPE_INVALID ) ) {
+                //TODO translate?
+                error_return = "Unmount failed: Unable to add arguments to DBus message";
+                dbus_message_unref( message );
+            } else {
+
+                dbus_error_init( &dbus_error );
+
+                reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                                   message,
+                                                                   -1,
+                                                                   &dbus_error );
+
+                dbus_message_unref( message );
+
+                if ( dbus_error_is_set( &dbus_error ) ) {
+                    error_return = AGUIXUtils::formatStringToString( catalog.getLocale( 971 ),
+                                                                     dbus_error.message );
+                    dbus_error_free( &dbus_error );
+                } else {
+                    unmount_failed = false;
+
+                    if ( reply ) {
+                        dbus_message_unref( reply );
+                    }
+                }
+            }
+        }
+    }
 #endif
     if ( unmount_failed ) return 1;
 
@@ -925,7 +1463,7 @@ int HWVolumeManager::cmdUnmount( HWVolume &vol, std::string &error_return )
     return 0;
 }
 
-int HWVolumeManager::halEject( HWVolume &vol, std::string &error_return )
+int HWVolumeManager::dbusEject( HWVolume &vol, std::string &error_return )
 {
     if ( vol.getUDI().empty() ) {
         error_return = catalog.getLocale( 923 );
@@ -1026,6 +1564,55 @@ int HWVolumeManager::halEject( HWVolume &vol, std::string &error_return )
             error_return = "Eject failed: No Eject method";
         }
     }
+#elif defined(HAVE_DBUS)
+    if ( HALreconnect() == 0 ) {
+        DBusMessage *message, *reply;
+        DBusError dbus_error;
+        const char *options[] = { NULL };
+        int num_options = 0;
+
+        message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                                vol.getUDI().c_str(),
+                                                "org.freedesktop.UDisks.Device",
+                                                "DriveEject" );
+
+        if ( message == NULL ) {
+            error_return = "Eject failed: Unable to create DBus message";
+        } else {
+
+            const char **options_p = options;
+            
+            if ( ! dbus_message_append_args( message, 
+                                             DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &options_p, num_options,
+                                             DBUS_TYPE_INVALID ) ) {
+                //TODO translate?
+                error_return = "Eject failed: Unable to add arguments to DBus message";
+                dbus_message_unref( message );
+            } else {
+
+                dbus_error_init( &dbus_error );
+
+                reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                                   message,
+                                                                   -1,
+                                                                   &dbus_error );
+
+                dbus_message_unref( message );
+
+                if ( dbus_error_is_set( &dbus_error ) ) {
+                    error_return = AGUIXUtils::formatStringToString( catalog.getLocale( 972 ),
+                                                                     dbus_error.message );
+                    dbus_error_free( &dbus_error );
+                } else {
+                    eject_failed = false;
+
+                    if ( reply ) {
+                        dbus_message_unref( reply );
+                    }
+                }
+            }
+        }
+    }
 #endif
     if ( eject_failed ) return 1;
 
@@ -1058,7 +1645,7 @@ int HWVolumeManager::cmdEject( HWVolume &vol, std::string &error_return )
     return 0;
 }
 
-int HWVolumeManager::halCloseTray( HWVolume &vol, std::string &error_return )
+int HWVolumeManager::dbusCloseTray( HWVolume &vol, std::string &error_return )
 {
     if ( vol.getUDI().empty() ) {
         error_return = catalog.getLocale( 924 );
@@ -1127,7 +1714,7 @@ int HWVolumeManager::halCloseTray( HWVolume &vol, std::string &error_return )
 #if 0
                 int hal_retcode;
 
-                // see comment in halMount
+                // see comment in dbusMount
                 if ( ! dbus_message_get_args( reply, 
                                               &error,
                                               DBUS_TYPE_UINT32, &hal_retcode,
@@ -1157,6 +1744,9 @@ int HWVolumeManager::halCloseTray( HWVolume &vol, std::string &error_return )
             error_return = "CloseTray failed: No CloseTray method";
         }
     }
+#elif defined(HAVE_DBUS)
+    //TODO implement once udisks supports closing the tray
+    error_return = "CloseTray failed: Not supported by udisks";
 #endif
     if ( closetray_failed ) return 1;
 
@@ -1263,6 +1853,36 @@ int HWVolumeManager::HALconnectionInit()
             m_dbus_conn = NULL;
         }
     }
+    return ( init_failed == true ) ? 1 : 0;
+#elif defined(HAVE_DBUS)
+    bool init_failed = false;
+    DBusError dbus_error;
+
+    if ( HALisConnected() ) return 0;
+
+    dbus_error_init( &dbus_error );
+
+    m_udisks_available = false;
+
+    m_dbus_conn = dbus_bus_get( DBUS_BUS_SYSTEM, &dbus_error );
+
+    if ( dbus_error_is_set( &dbus_error ) ) {
+        fprintf( stderr, "DBUS: error %s\n", dbus_error.message );
+        dbus_error_free( &dbus_error );
+        init_failed = true;
+    }else if ( m_dbus_conn == NULL ){
+        fprintf( stderr, "Couldn't connect to system bus" );
+        init_failed = true;
+    } else {
+        try {
+            if ( getStringProperty( "/org/freedesktop/UDisks",
+                                    "DaemonVersion" ) != "" ) {
+                m_udisks_available = true;
+            }
+        } catch ( int ) {
+        }
+    }
+
     return ( init_failed == true ) ? 1 : 0;
 #else
     return 0;
@@ -1292,6 +1912,12 @@ int HWVolumeManager::HALconnectionFini()
         dbus_connection_unref( m_dbus_conn );
     }
     return 0;
+#elif defined(HAVE_DBUS)
+    if ( m_dbus_conn != NULL ) {
+        dbus_connection_unref( m_dbus_conn );
+        m_dbus_conn = NULL;
+    }
+    return 0;
 #else
     return 0;
 #endif
@@ -1306,6 +1932,13 @@ bool HWVolumeManager::HALisConnected()
         }
     }
     return false;
+#elif defined(HAVE_DBUS)
+    if ( m_dbus_conn != NULL ) {
+        if ( dbus_connection_get_is_connected( m_dbus_conn ) ) {
+            return true;
+        }
+    }
+    return false;
 #else
     return false;
 #endif
@@ -1320,6 +1953,14 @@ int HWVolumeManager::HALreconnect()
 
     if ( HALconnectionInit() != 0 ) return 1;
 
+    return 0;
+#elif defined(HAVE_DBUS)
+    if ( HALisConnected() ) return 0;
+
+    if ( HALconnectionFini() != 0 ) return 1;
+
+    if ( HALconnectionInit() != 0 ) return 1;
+
     return 0;
 #else
     return 0;
@@ -1402,6 +2043,89 @@ std::list<HWVolume> HWVolumeManager::getEjectableDevices()
         }
         dbus_error_init( &error );
     }
+#elif defined(HAVE_DBUS)
+    if ( HALreconnect() == 0 && m_udisks_available)  {
+        DBusMessage *message, *reply;
+        DBusMessageIter reply_iter, sub_iter;
+        DBusError dbus_error;
+
+        message = dbus_message_new_method_call( "org.freedesktop.UDisks",
+                                                "/org/freedesktop/UDisks",
+                                                "org.freedesktop.UDisks",
+                                                "EnumerateDevices" );
+
+        if ( message == NULL ) {
+        } else {
+
+            dbus_error_init( &dbus_error );
+
+            reply = dbus_connection_send_with_reply_and_block( m_dbus_conn,
+                                                               message,
+                                                               -1,
+                                                               &dbus_error );
+
+            dbus_message_unref( message );
+
+            if ( dbus_error_is_set( &dbus_error ) ) {
+                fprintf( stderr, "Couldn't enumerate udisks devices: %s\n", dbus_error.message );
+                dbus_error_free( &dbus_error );
+            } else if ( reply == NULL ) {
+                fprintf( stderr, "Couldn't enumerate udisks devices: %s\n", dbus_error.message );
+            } else {
+
+                dbus_message_iter_init( reply, &reply_iter );
+
+                if ( dbus_message_iter_get_arg_type( &reply_iter ) ==
+                     DBUS_TYPE_ARRAY ) {
+
+                    dbus_message_iter_recurse( &reply_iter,
+                                               &sub_iter );
+
+                    while ( dbus_message_iter_get_arg_type( &sub_iter ) ==
+                            DBUS_TYPE_OBJECT_PATH ) {
+                        const char *udi = NULL;
+
+                        dbus_message_iter_get_basic( &sub_iter, &udi );
+
+                        if ( udi ) {
+                            bool use_volume = false;
+
+                            try {
+                                if ( getBoolProperty( udi, "DriveIsMediaEjectable" ) ) {
+                                    use_volume = true;
+                                }
+                            } catch ( int ) {
+                            }
+
+                            // first check whether we should ignore the volume
+                            try {
+                                if ( getBoolProperty( udi, "DeviceIsSystemInternal" ) ) {
+                                    use_volume = false;
+                                }
+                            } catch ( int ) {
+                            }
+
+                            if ( use_volume ) {
+                                HWVolume v( udi );
+
+                                update( v );
+
+                                if ( managed.count( getOSDevice( v.getDevice() ) ) < 1 ) {
+                                    managed.insert( getOSDevice( v.getDevice() ) );
+                
+                                    l.push_back( v );
+                                }
+                            }
+                        }
+
+                        dbus_message_iter_next( &sub_iter );
+                    }
+                }
+
+                dbus_message_unref( reply );
+            }
+        }
+    }
 #endif
 
     return l;
@@ -1446,6 +2170,33 @@ std::string HWVolumeManager::getDescription( const HWVolume &vol )
             dbus_error_free( &error );
         }
     }
+#elif defined(HAVE_DBUS)
+    if ( HALreconnect() == 0 ) {
+
+        std::string vendor, product;
+
+        try {
+            vendor = getStringProperty( vol.getUDI().c_str(), "DriveVendor" );
+        } catch ( int ) {
+        }
+        try {
+            product = getStringProperty( vol.getUDI().c_str(), "DriveModel" );
+        } catch ( int ) {
+        }
+
+        if ( ! vendor.empty() ) {
+            descr = vendor;
+            descr += " ";
+        }
+        if ( ! product.empty() ) {
+            descr += product;
+        }
+    }
 #endif
     return descr;
 }
+
+bool HWVolumeManager::getUDisksAvailable() const
+{
+    return m_udisks_available;
+}
