@@ -28,6 +28,10 @@
 #include "wchotkey.hh"
 #include "wcbutton.hh"
 #include "workerversion.h"
+#include "wconfig.h"
+#include "simplelist.hh"
+#include "datei.h"
+#include "worker_locale.h"
 
 void WConfig::setDefaultConfig()
 {
