@@ -297,6 +297,7 @@ public:
                            const mode_t wanted_mode,
                            struct NM_copyorder *copyorder );
     NM_specialsourceExt *getSpecialsourceForFE( const FileEntry *searchfe );
+    NM_specialsourceExt *getSpecialsourceForFilename( const std::string &filename );
 
     void activateEntry( const std::string &name );
 
@@ -649,6 +650,7 @@ protected:
         std::auto_ptr<FileEntry> entry_for_popup;
         PopUpMenu *lv_popup_menu;
         int m_tab;
+        std::map< int, bool > entry_is_action_submenu;
     };
 
     PopUpSettings m_current_popup_settings;
@@ -687,7 +689,6 @@ protected:
 
     std::string m_last_completion;
 
-    std::string convertMatchStringToFlexibleMatch( const std::string &str );
     std::string cleanFlexibleMatchString( const std::string &str );
 
     bool m_flexible_matching_active;
@@ -700,6 +701,8 @@ protected:
     } m_cached_values;
 
     NM_Filetype_Thread m_ft_thread;
+
+    int prepareContext( class ActionMessage &amsg );
 };
 
 #endif
