@@ -230,7 +230,26 @@ void NormalMode::copy( struct NM_copyorder *copyorder )
         if ( ss1->row >= 0 ) {
             lv->setVisMark( ss1->row, true );
             lv->showRow( ss1->row, false );
-            lv->timeLimitedRedraw();
+
+            // this is some optimization when copying a lot of small files
+            // limit redrawing in this case
+            if ( tcod == NULL ) {
+                loff_t size = 0;
+
+                if ( ss1->entry()->isLink && ! ss1->entry()->isCorrupt ) {
+                    size = ss1->entry()->dsize();
+                } else {
+                    size = ss1->entry()->size();
+                }
+
+                if ( size > 128 * 1024 ) {
+                    lv->redraw();
+                } else {
+                    lv->timeLimitedRedraw();
+                }
+            } else {
+                lv->redraw();
+            }
         }
         if(tcod!=NULL) {
             ce1 = copydir( ss1->entry(), tcod, copyorder, true, copyorder->destdir );
