@@ -1,6 +1,6 @@
 /* scriptop.cc
- * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2002-2006 Ralf Hoffmann.
+ * This file belongs to Worker, a file manager for UN*X/X11.
+ * Copyright (C) 2002-2006,2011 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -26,6 +26,13 @@
 #include "wpucontext.h"
 #include "execlass.h"
 #include <aguix/acontainerbb.h>
+#include "worker_locale.h"
+#include <aguix/fieldlistview.h>
+#include <aguix/choosebutton.h>
+#include <aguix/stringgadget.h>
+#include <aguix/button.h>
+#include <aguix/cyclebutton.h>
+#include "datei.h"
 
 const char *ScriptOp::name="ScriptOp";
 
@@ -51,6 +58,7 @@ ScriptOp::ScriptOp() : FunctionProto()
   wintext = NULL;
   wintext_useoutput = false;
   hasConfigure = true;
+  m_category = FunctionProto::CAT_SCRIPTING;
 }
 
 ScriptOp::~ScriptOp()
@@ -103,7 +111,7 @@ ScriptOp::getName()
 int
 ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
 {
-  char *tstr, *tstr2, *tstr3;
+  char *tstr2, *tstr3;
   int erg, l, exeerror = 1;
   
   switch ( type ) {
@@ -113,12 +121,14 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
          3.push string to stack nr
        */
       if ( push_string != NULL ) {
-        tstr = wpu->parse( push_string, a_max( EXE_STRING_LEN - 1024, 256 ), ( ( push_useoutput == true ) ? true : false ) );
-        if ( tstr != NULL ) {
+        std::string res_str;
+
+        if ( wpu->parse( push_string, res_str, a_max( EXE_STRING_LEN - 1024, 256 ),
+                         ( ( push_useoutput == true ) ? true : false ) ) == NormalMode::PARSE_SUCCESS ) {
           if ( push_useoutput == true ) {
             std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL, msg->getWorker() ) );
             ec->cdDir( wpu->getBaseDir() );
-            ec->addCommand( "%s", tstr );
+            ec->addCommand( "%s", res_str.c_str() );
             tstr2 = ec->getOutput( &exeerror );
             
             if ( exeerror == 0 ) {
@@ -143,8 +153,8 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
                 char *reqstr;
                 std::string error, str1;
                 
-                reqstr = (char*)_allocsafe( strlen( catalog.getLocale( 750 ) ) + strlen( tstr ) + 1 );
-                sprintf( reqstr, catalog.getLocale( 750 ), tstr );
+                reqstr = (char*)_allocsafe( strlen( catalog.getLocale( 750 ) ) + res_str.size() + 1 );
+                sprintf( reqstr, catalog.getLocale( 750 ), res_str.c_str() );
                 
                 ec->readErrorOutput( error, 1024 );
                 
@@ -164,16 +174,15 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
               Requester *req;
               req = new Requester( Worker::getAGUIX() );
               tstr3 = (char*)_allocsafe( strlen( catalog.getLocale( 485 ) ) +
-                                         strlen( tstr ) +
+                                         res_str.size() +
                                          A_BYTESFORNUMBER( stack_nr ) + 1 );
-              sprintf( tstr3, catalog.getLocale( 485 ), tstr, stack_nr );
+              sprintf( tstr3, catalog.getLocale( 485 ), res_str.c_str(), stack_nr );
               req->request( catalog.getLocale( 124 ), tstr3, catalog.getLocale( 11 ) );
               _freesafe( tstr3 );
               delete req;
             }
-            wpu->push( stack_nr, tstr );
+            wpu->push( stack_nr, res_str );
           }
-          _freesafe( tstr );
         }
       }
       break;
@@ -187,9 +196,9 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
         if ( dodebug == true ) {
           Requester *req;
           req = new Requester( Worker::getAGUIX() );
-          tstr = (char*)_allocsafe( strlen( catalog.getLocale( 484 ) ) +
-                                    strlen( if_test ) +
-                                    A_BYTESFORNUMBER( erg ) + 1 );
+          char *tstr = (char*)_allocsafe( strlen( catalog.getLocale( 484 ) ) +
+                                          strlen( if_test ) +
+                                          A_BYTESFORNUMBER( erg ) + 1 );
           sprintf( tstr, catalog.getLocale( 484 ), if_test, erg );
           req->request( catalog.getLocale( 124 ), tstr, catalog.getLocale( 11 ) );
           _freesafe( tstr );
@@ -199,9 +208,9 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
           if ( wpu->continueAtLabel( if_label ) != 0 ) {
             Requester *req;
             req = new Requester( Worker::getAGUIX() );
-            tstr = (char*)_allocsafe( strlen( catalog.getLocale( 459 ) ) +
-                                      strlen( if_label ) +
-                                      A_BYTESFORNUMBER( l ) + 1 );
+            char *tstr = (char*)_allocsafe( strlen( catalog.getLocale( 459 ) ) +
+                                            strlen( if_label ) +
+                                            A_BYTESFORNUMBER( l ) + 1 );
             sprintf( tstr, catalog.getLocale( 459 ), if_label, l + 1 );
             req->request( catalog.getLocale( 347 ), tstr, catalog.getLocale( 11 ) );
             _freesafe( tstr );
@@ -211,7 +220,7 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
         } else {
           Requester *req;
           req = new Requester( Worker::getAGUIX() );
-          tstr = (char*)_allocsafe( strlen( catalog.getLocale( 460 ) ) + A_BYTESFORNUMBER( l ) + 1 );
+          char *tstr = (char*)_allocsafe( strlen( catalog.getLocale( 460 ) ) + A_BYTESFORNUMBER( l ) + 1 );
           sprintf( tstr, catalog.getLocale( 460 ), l + 1 );
           req->request( catalog.getLocale( 347 ), tstr, catalog.getLocale( 11 ) );
           _freesafe( tstr );
@@ -221,7 +230,7 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
       } else {
         Requester *req;
         req = new Requester( Worker::getAGUIX() );
-        tstr = (char*)_allocsafe( strlen( catalog.getLocale( 460 ) ) + A_BYTESFORNUMBER( l ) + 1 );
+        char *tstr = (char*)_allocsafe( strlen( catalog.getLocale( 460 ) ) + A_BYTESFORNUMBER( l ) + 1 );
         sprintf( tstr, catalog.getLocale( 460 ), l + 1 );
         req->request( catalog.getLocale( 347 ), tstr, catalog.getLocale( 11 ) );
         _freesafe( tstr );
@@ -253,20 +262,21 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
       }
       if ( change_progress == true ) {
         if ( progress != NULL ) {
-          tstr = wpu->parse( progress, a_max( EXE_STRING_LEN - 1024, 256 ), ( ( progress_useoutput == true ) ? true : false ) );
-          if ( tstr != NULL ) {
+          std::string res_str;
+          if ( wpu->parse( progress, res_str, a_max( EXE_STRING_LEN - 1024, 256 ),
+                           ( ( progress_useoutput == true ) ? true : false ) ) == NormalMode::PARSE_SUCCESS ) {
             if ( progress_useoutput == true ) {
               std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
               ec->cdDir( wpu->getBaseDir() );
-              ec->addCommand( "%s", tstr );
+              ec->addCommand( "%s", res_str.c_str() );
               tstr2 = ec->getOutput( &exeerror );
               
               if ( exeerror == 0 ) {
-                _freesafe( tstr );
                 if ( tstr2 != NULL ) {
-                  tstr = tstr2;
+                  res_str = tstr2;
+                  _freesafe( tstr2 );
                 } else {
-                  tstr = NULL;
+                  res_str = "";
                 }
               } else {
                 if ( tstr2 != NULL ) _freesafe( tstr2 );
@@ -274,8 +284,8 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
                   char *reqstr;
                   std::string error, str1;
                   
-                  reqstr = (char*)_allocsafe( strlen( catalog.getLocale( 751 ) ) + strlen( tstr ) + 1 );
-                  sprintf( reqstr, catalog.getLocale( 751 ), tstr );
+                  reqstr = (char*)_allocsafe( strlen( catalog.getLocale( 751 ) ) + res_str.size() + 1 );
+                  sprintf( reqstr, catalog.getLocale( 751 ), res_str.c_str() );
                   
                   ec->readErrorOutput( error, 1024 );
                   
@@ -288,35 +298,36 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
                   
                   Worker::getRequester()->request( catalog.getLocale( 347 ), ts, catalog.getLocale( 11 ) );
                 }
-                _freesafe( tstr );
-                tstr = NULL;
+                res_str = "";
               }
             }
-            if ( tstr != NULL ) {
-              l = atoi( tstr );
-              _freesafe( tstr );
-              if ( ( l < 0 ) || ( l > 100 ) ) l = 0;
-              wpu->setWinProgress( l );
+            if ( ! res_str.empty() ) {
+                l = atoi( res_str.c_str() );
+                if ( l < 0 || l > 100 ) l = 0;
+                wpu->setWinProgress( l );
             }
           }
         }
       }
       if ( change_text == true ) {
         if ( wintext != NULL ) {
-          tstr = wpu->parse( wintext, a_max( EXE_STRING_LEN - 1024, 256 ), ( ( wintext_useoutput == true ) ? true : false ) );
-          if ( tstr != NULL ) {
+          std::string res_str;
+          bool valid = true;
+          if ( wpu->parse( wintext, res_str, a_max( EXE_STRING_LEN - 1024, 256 ),
+                           ( ( wintext_useoutput == true ) ? true : false ) ) == NormalMode::PARSE_SUCCESS ) {
             if ( wintext_useoutput == true ) {
               std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
               ec->cdDir( wpu->getBaseDir() );
-              ec->addCommand( "%s", tstr );
+              ec->addCommand( "%s", res_str.c_str() );
               tstr2 = ec->getOutput( &exeerror );
               
               if ( exeerror == 0 ) {
-                _freesafe( tstr );
                 if ( tstr2 != NULL ) {
-                  tstr = tstr2;
+                  res_str = tstr2;
+                  _freesafe( tstr2 );
                 } else {
-                  tstr = NULL;
+                  res_str = "";
+                  valid = false;
                 }
               } else {
                 if ( tstr2 != NULL ) _freesafe( tstr2 );
@@ -324,8 +335,8 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
                   char *reqstr;
                   std::string error, str1;
                   
-                  reqstr = (char*)_allocsafe( strlen( catalog.getLocale( 751 ) ) + strlen( tstr ) + 1 );
-                  sprintf( reqstr, catalog.getLocale( 751 ), tstr );
+                  reqstr = (char*)_allocsafe( strlen( catalog.getLocale( 751 ) ) + res_str.size() + 1 );
+                  sprintf( reqstr, catalog.getLocale( 751 ), res_str.c_str() );
                   
                   ec->readErrorOutput( error, 1024 );
                   
@@ -338,13 +349,12 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
                   
                   Worker::getRequester()->request( catalog.getLocale( 347 ), ts, catalog.getLocale( 11 ) );
                 }
-                _freesafe( tstr );
-                tstr = NULL;
+                res_str = "";
+                valid = false;
               }
             }
-            if ( tstr != NULL ) {
-              wpu->setWinText( tstr );
-              _freesafe( tstr );
+            if ( valid ) {
+                wpu->setWinText( res_str.c_str() );
             }
           }
         }
@@ -356,9 +366,9 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
           l = wpu->getCurrentLine();
           Requester *req;
           req = new Requester( Worker::getAGUIX() );
-          tstr = (char*)_allocsafe( strlen( catalog.getLocale( 514 ) ) +
-                                    strlen( label ) +
-                                    A_BYTESFORNUMBER( l ) + 1 );
+          char *tstr = (char*)_allocsafe( strlen( catalog.getLocale( 514 ) ) +
+                                          strlen( label ) +
+                                          A_BYTESFORNUMBER( l ) + 1 );
           sprintf( tstr, catalog.getLocale( 514 ), label, l + 1 );
           req->request( catalog.getLocale( 347 ), tstr, catalog.getLocale( 11 ) );
           _freesafe( tstr );
