@@ -1,6 +1,6 @@
 /* setfilterop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2010 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2008,2010-2011 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -28,11 +28,20 @@
 #include <algorithm>
 #include "workerinitialsettings.hh"
 #include "nwc_path.hh"
+#include "pers_string_list.hh"
+#include "datei.h"
+#include "worker_locale.h"
+#include <aguix/cyclebutton.h>
+#include <aguix/fieldlistview.h>
+#include <aguix/choosebutton.h>
+#include <aguix/stringgadget.h>
+#include <aguix/button.h>
+#include "wconfig.h"
 
 const char *SetFilterOp::name="SetFilterOp";
 int SetFilterOp::instance_counter = 0;
-List *SetFilterOp::filter_history = NULL;
-int SetFilterOp::history_size = 30;
+size_t SetFilterOp::history_size = 30;
+std::auto_ptr< PersistentStringList > SetFilterOp::m_history;
 
 class SetFilterCB : public GenericCallbackArg<void, AGMessage*>
 {
@@ -67,7 +76,7 @@ SetFilterOp::SetFilterOp() : FunctionProto()
 {
   filtermode=INCLUDE_FILTER;
   request_flags=false;
-  filter=dupstring("");
+  filter = "";
 
   if ( instance_counter == 0 ) {
     initHistory();
@@ -92,14 +101,14 @@ SetFilterOp::SetFilterOp() : FunctionProto()
 
   m_popup_menu = NULL;
   m_worker = NULL;
+
+    m_category = FunctionProto::CAT_FILELIST;
 }
 
 SetFilterOp::~SetFilterOp()
 {
     cleanUpPopUp();
 
-  _freesafe(filter);
-
   instance_counter--;
   if ( instance_counter == 0 ) {
     closeHistory();
@@ -112,8 +121,7 @@ SetFilterOp::duplicate() const
   SetFilterOp *ta = new SetFilterOp();
   ta->filtermode=filtermode;
   ta->request_flags=request_flags;
-  _freesafe(ta->filter);
-  ta->filter=dupstring(filter);
+  ta->filter = filter;
 
   ta->m_bookmark_label = m_bookmark_label;
   ta->m_bookmark_filter = m_bookmark_filter;
@@ -173,7 +181,7 @@ SetFilterOp::save(Datei *fh)
       fh->configPutPair( "mode", "include" );
       break;
   }
-  fh->configPutPairString( "filter", filter );
+  fh->configPutPairString( "filter", filter.c_str() );
 
   fh->configPutPairString( "bookmarklabel", m_bookmark_label.c_str() );
   switch ( m_bookmark_filter ) {
@@ -230,7 +238,7 @@ SetFilterOp::normalmodesf( ActionMessage *am )
     } else {
         // set values in t* variables
         tfiltermode = filtermode;
-        tfilter = dupstring( filter );
+        tfilter = filter;
         
         m_t_bookmark_label = m_bookmark_label;
         m_t_bookmark_filter = m_bookmark_filter;
@@ -245,34 +253,34 @@ SetFilterOp::normalmodesf( ActionMessage *am )
             if ( tfiltermode == UNSET_ALL ) {
                 nm1->unsetAllFilters();
             } else {
-                if ( strlen( tfilter ) > 0 ) {
+                if ( ! tfilter.empty() ) {
                     switch ( tfiltermode ) {
                         case INCLUDE_FILTER:
                             try {
-                                NormalMode::nm_filter_t t1 = nm1->checkFilter( tfilter );
+                                NormalMode::nm_filter_t t1 = nm1->checkFilter( tfilter.c_str() );
                                 if ( t1 != NormalMode::NM_FILTER_INCLUDE ) {
-                                    nm1->setFilter( tfilter, NormalMode::NM_FILTER_INCLUDE );
+                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_INCLUDE );
                                 } else {
-                                    nm1->setFilter( tfilter, NormalMode::NM_FILTER_UNSET );
+                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_UNSET );
                                 }
                             } catch ( int ) {
-                                nm1->setFilter( tfilter, NormalMode::NM_FILTER_INCLUDE );
+                                nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_INCLUDE );
                             }
                             break;
                         case EXCLUDE_FILTER:
                             try {
-                                NormalMode::nm_filter_t t1 = nm1->checkFilter( tfilter );
+                                NormalMode::nm_filter_t t1 = nm1->checkFilter( tfilter.c_str() );
                                 if ( t1 != NormalMode::NM_FILTER_EXCLUDE ) {
-                                    nm1->setFilter( tfilter, NormalMode::NM_FILTER_EXCLUDE );
+                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_EXCLUDE );
                                 } else {
-                                    nm1->setFilter( tfilter, NormalMode::NM_FILTER_UNSET );
+                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_UNSET );
                                 }
                             } catch ( int ) {
-                                nm1->setFilter( tfilter, NormalMode::NM_FILTER_EXCLUDE );
+                                nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_EXCLUDE );
                             }
                             break;
                         case UNSET_FILTER:
-                            nm1->setFilter( tfilter, NormalMode::NM_FILTER_UNSET );
+                            nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_UNSET );
                             break;
                         default:
                             break;
@@ -304,8 +312,6 @@ SetFilterOp::normalmodesf( ActionMessage *am )
                 }
             }
         }
-
-        _freesafe( tfilter );
     }
     return 0;
 }
@@ -313,10 +319,6 @@ SetFilterOp::normalmodesf( ActionMessage *am )
 int
 SetFilterOp::doconfigure(int mode)
 {
-  if ( filter_history == NULL ) return false;  // this can only happen when the stupid
-                                               // programmer calls closeHistory
-                                               // but to be sure...
-
   AGUIX *aguix = Worker::getAGUIX();
   StringGadget *sg;
   FieldListView *lv;
@@ -327,7 +329,7 @@ SetFilterOp::doconfigure(int mode)
   int endmode=-1;
   char *tstr;
   int i;
-  int id, row;
+  int row;
   bool lvstill;
   int lastrow;
   const int cincw = AContainer::ACONT_MINH +
@@ -421,20 +423,19 @@ SetFilterOp::doconfigure(int mode)
     lv->setDisplayFocus( true );
     lv->setAcceptFocus( true );
   
-    id = filter_history->initEnum();
-    tstr = (char*)filter_history->getFirstElement( id );
-    while ( tstr != NULL ) {
-      row = lv->insertRow( 0 );
-      lv->setText( row, 0, tstr );
-      lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
-      tstr = (char*)filter_history->getNextElement( id );
+    std::list< std::string > cur_history = m_history->getList();
+    for ( std::list< std::string >::const_iterator it1 = cur_history.begin();
+          it1 != cur_history.end();
+          it1++ ) {
+        row = lv->insertRow( 0 );
+        lv->setText( row, 0, *it1 );
+        lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
     }
-    filter_history->closeEnum( id );
-    lv->showRow( filter_history->size() - 1 );
+    lv->showRow( cur_history.size() - 1 );
     lv->redraw();
   } else lv = NULL;
 
-  sg = (StringGadget*)ac1_2->add( new StringGadget( aguix, 0, 0, 100, filter, 0 ), 2, 1, cincw );
+  sg = (StringGadget*)ac1_2->add( new StringGadget( aguix, 0, 0, 100, filter.c_str(), 0 ), 2, 1, cincw );
 
   ac1_2->setMinWidth( 5, 1, 0 );
   ac1_2->setMaxWidth( 5, 1, 0 );
@@ -588,10 +589,7 @@ SetFilterOp::doconfigure(int mode)
                     lv->takeFocus();
                     lv->showActive();
                     lastrow = lv->getActiveRow();
-                    tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 - lastrow );
-                    if ( tstr != NULL ) {
-                      sg->setText( tstr );
-                    }
+                    sg->setText( lv->getText( lastrow, 0 ).c_str() );
                   }
                 }
                 lvstill = false;
@@ -629,10 +627,7 @@ SetFilterOp::doconfigure(int mode)
                 else lvstill = false;
                 lastrow = row;
                 if ( lv->isValidRow( row ) == true ) {
-                    tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 - row );
-                    if ( tstr != NULL ) {
-                        sg->setText( tstr );
-                    }
+                    sg->setText( lv->getText( row, 0 ).c_str() );
                 }
             }
           break;
@@ -645,7 +640,7 @@ SetFilterOp::doconfigure(int mode)
     // ok
     if(mode==1) {
       // store in t-variables
-      tfilter=dupstring(sg->getText());
+      tfilter = sg->getText();
       addHistoryItem( tfilter );
       switch(fmcyb->getSelectedOption()) {
         case 1:
@@ -688,8 +683,7 @@ SetFilterOp::doconfigure(int mode)
     } else {
       // store in normal variables
       request_flags = rfcb->getState();
-      _freesafe(filter);
-      filter=dupstring(sg->getText());
+      filter = sg->getText();
       switch(fmcyb->getSelectedOption()) {
         case 1:
           filtermode=EXCLUDE_FILTER;
@@ -744,87 +738,31 @@ SetFilterOp::configure()
 
 void SetFilterOp::initHistory()
 {
-  if ( filter_history != NULL ) return;
-  char *tstr;
-  Datei *fp = new Datei();
-  std::string str1;
+    std::string str1;
 
-  filter_history = new List();
-
-  str1 = WorkerInitialSettings::getInstance().getConfigBaseDir();
-  str1 = NWC::Path::join( str1, "setfilter-history" );
-  fp->open( str1.c_str(), "r" );
-  
-  while ( fp->isEOF() == false ) {
-      tstr = fp->getLine();
-      if ( tstr == NULL ) break;
-      if ( strlen( tstr ) > 0 ) {
-        filter_history->addElement( tstr );
-      } else {
-        _freesafe( tstr );
-      }
-  }
-  
-  fp->close();
-  delete fp;
+    str1 = WorkerInitialSettings::getInstance().getConfigBaseDir();
+    str1 = NWC::Path::join( str1, "setfilter-history" );
+    m_history = std::auto_ptr< PersistentStringList >( new PersistentStringList( str1 ) );
 }
 
 void SetFilterOp::closeHistory()
 {
-  if ( filter_history == NULL ) return;
-  char *tstr;
-  int id;
-  Datei *fp = new Datei();
-  std::string str1;
-
-  str1 = WorkerInitialSettings::getInstance().getConfigBaseDir();
-  str1 = NWC::Path::join( str1, "setfilter-history" );
-  fp->open( str1.c_str(), "w" );
-  
-  id = filter_history->initEnum();
-  tstr = (char*)filter_history->getFirstElement( id );
-  while ( tstr != NULL ) {
-    fp->putLine( tstr );
-    _freesafe( tstr );
-    filter_history->removeFirstElement();
-    tstr = (char*)filter_history->getFirstElement( id );
-  }
-  filter_history->closeEnum( id );
-  delete filter_history;
-  filter_history = NULL;
-  
-  fp->close();
-  delete fp;
+    m_history.reset();
 }
 
-void SetFilterOp::addHistoryItem( const char *str )
+void SetFilterOp::addHistoryItem( const std::string &str )
 {
-  char *tstr;
-  int id;
-  
-  id = filter_history->initEnum();
-  tstr = (char*)filter_history->getFirstElement( id );
-  while ( tstr != NULL ) {
-    if ( strcmp( str, tstr ) == 0 ) break;
-    tstr = (char*)filter_history->getNextElement( id );
-  }
-  filter_history->closeEnum( id );
+    if ( str.empty() ) return;
 
-  if ( tstr == NULL ) {
-    // not already in the list
-    if ( filter_history->size() >= history_size ) {
-      tstr = (char*)filter_history->getElementAt( filter_history->size() - 1 );
-      if ( tstr != NULL ) {
-        _freesafe( tstr );
-      }
-      filter_history->removeElementAt( filter_history->size() - 1 );
+    if ( m_history->contains( str ) ) {
+        m_history->removeEntry( str );
+        m_history->pushFrontEntry( str );
+    } else {
+        if ( m_history->size() >= history_size ) {
+            m_history->removeLast();
+        }
+        m_history->pushFrontEntry( str );
     }
-    filter_history->addElementAt( 0, dupstring( str ) );
-  } else {
-    // in the list so put it at the beginning
-    filter_history->removeElement( tstr );
-    filter_history->addElementAt( 0, tstr );
-  }
 }
 
 void SetFilterOp::setRequestFlags( bool nv )
@@ -835,8 +773,7 @@ void SetFilterOp::setRequestFlags( bool nv )
 void SetFilterOp::setFilter( const char *nv )
 {
   if ( nv == NULL ) return;
-  _freesafe( filter );
-  filter = dupstring( nv );
+  filter = nv;
 }
 
 void SetFilterOp::setFiltermode( setfilterop_mode nv )
