@@ -195,7 +195,7 @@ StartProgOp::normalmodestartprog( ActionMessage *am )
           tmpoutput_lnk = Datei::createTMPHardlink( tmpoutput );
           pea = new ProcessExitAction( true,
                                        tmpoutput_lnk,
-                                       am->getWorker() );
+                                       am->getWorker(), NULL );
       }
 
       Datei fp;
