@@ -1,6 +1,6 @@
 /* volumemanagerop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009 Ralf Hoffmann.
+ * Copyright (C) 2009,2011 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -25,6 +25,9 @@
 #include "volumemanagerui.hh"
 #include "nwc_fsentry.hh"
 #include "nmspecialsourceext.hh"
+#include "worker_locale.h"
+#include "worker.h"
+#include "fileentry.hh"
 
 const char *VolumeManagerOp::name = "VolumeManagerOp";
 
