@@ -1,6 +1,6 @@
 /* volumemanagerui.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009-2011 Ralf Hoffmann.
+ * Copyright (C) 2009-2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -333,15 +333,15 @@ void VolumeManagerUI::maximizeWin()
     int old_w = m_lv->getWidth();
     int old_h = m_lv->getHeight();
 
-    m_lv->maximizeX();
-    m_lv->maximizeY();
+    int new_w = m_lv->getMaximumWidth();
+    int new_h = m_lv->getMaximumHeight();
 
-    if ( m_lv->getWidth() <= old_w &&
-         m_lv->getHeight() <= old_h ) {
+    if ( new_w <= old_w &&
+         new_h <= old_h ) {
         m_cont2->rearrange();
     } else {
-        int my_w = m_lv->getWidth() + 10;
-        int my_h = m_lv->getHeight() + 10;
+        int my_w = new_w + 10;
+        int my_h = new_h + 10;
 
         if ( my_w < 400 ) my_w = 400;
         if ( my_h < 300 ) my_h = 300;
@@ -351,15 +351,15 @@ void VolumeManagerUI::maximizeWin()
         m_cont2->resize( mw, mh );
         m_cont2->rearrange();
 
-        if ( my_w < m_lv->getWidth() ) {
+        if ( my_w < new_w ) {
             m_cont2->setMinWidth( my_w, 0, 0 );
         } else {
-            m_cont2->setMinWidth( m_lv->getWidth(), 0, 0 );
+            m_cont2->setMinWidth( new_w, 0, 0 );
         }
-        if ( my_h < m_lv->getHeight() ) {
+        if ( my_h < new_h ) {
             m_cont2->setMinHeight( my_h, 0, 0 );
         } else {
-            m_cont2->setMinHeight( m_lv->getHeight(), 0, 0 );
+            m_cont2->setMinHeight( new_h, 0, 0 );
         }
         m_win->contMaximize( true );
     }
@@ -419,7 +419,7 @@ void VolumeManagerUI::updateLV()
             m_row_to_volume[row] = *it1;
 
             if ( volumeInList( *it1, nl ) ) {
-                m_lv->setVisMark( row, true );
+                m_lv->setVisMarkQ( row, true );
             }
 
             if ( cur_dev.empty() == false &&
