@@ -1,6 +1,6 @@
 /* wconfig.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -177,8 +177,7 @@ public:
   void setShowHeader( int side, bool nv );
   bool getShowHeader( int side ) const;
 
-  int getDatelen( const struct tm *timeptr );
-  int writeDateToString( char *buf, int maxlen, struct tm *timeptr );
+  int writeDateToString( std::string &str, struct tm *timeptr );
   
   bool checkForNewerConfig();
   time_t getFileConfigMod();
@@ -336,6 +335,9 @@ public:
     std::string getVMEjectCommand() const;
     void setVMCloseTrayCommand( const std::string &cmd );
     std::string getVMCloseTrayCommand() const;
+
+    void setPathJumpAllowDirs( const std::list< std::string > &l );
+    const std::list< std::string > &getPathJumpAllowDirs();
 protected:
   char *lang;
   unsigned int rows,columns;
@@ -493,6 +495,8 @@ protected:
     } m_vm_settings;
 
     bool m_use_version_string_compare;
+
+    std::list< std::string > m_pathjump_allow_dirs;
 };
 
 extern WConfig *wconfig;
