@@ -36,10 +36,9 @@ ClockPanel::~ClockPanel()
 
 int ClockPanel::create()
 {
-  int res;
   char *tstr;
   
-  res = Panel::create();
+  Panel::create();
 
   AContainer *ac1 = setContainer( new AContainer( this, 1, 4 ), true );
   ac1->setBorderWidth( 5 );
