@@ -41,9 +41,7 @@ DCDPanel::~DCDPanel()
 
 int DCDPanel::create()
 {
-  int res;
-  
-  res = Panel::create();
+  Panel::create();
 
   int trow;
   int pos;
