@@ -1,6 +1,6 @@
 /* worker_locale.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2012 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -27,8 +27,8 @@
 #include "nwc_path.hh"
 #include "datei.h"
 
-#define CATENTRY 970
-#define CATCOMENTRY 60
+#define CATENTRY 980
+#define CATCOMENTRY 62
 #define CATFLAGENTRY 59
 
 WorkerLocale catalog;
@@ -1072,6 +1072,16 @@ match any font." );
   catalog[967]=dupstring( "Go to entry but keep window" );
   catalog[968]=dupstring( "Creating temporary file: %3.0f%% done..." );
   catalog[969]=dupstring( "Creating temporary file: %3.0f%% done (total: %3.0f%%)..." );
+  catalog[970]=dupstring( "additional actions defined for current entry" );
+  catalog[971]=dupstring( "Unmount failed: %s" );
+  catalog[972]=dupstring( "Eject failed: %s" );
+  catalog[973]=dupstring( "%s (%ld:%02ld at current rate)" );
+  catalog[974]=dupstring( "Path jump" );
+  catalog[975]=dupstring( "List of allowed directories to track for path jump command.|All those directories and their sub-directories are allowed to store in a local database|to allow faster access to often used directories. It's a prefix comparison|so add a trailing slash if necessary to avoid mismatches." );
+  catalog[976]=dupstring( "Enter directory to allow to be tracked by the path jump command:" );
+  catalog[977]=dupstring( "Also show entries from bookmarks and persistant path store (Ctrl+a)" );
+  catalog[978]=dupstring( "Match quality" );
+  catalog[979]=dupstring( "String copied to clipboard:" );
 
 //  catalog[]=dupstring( "" );
   
@@ -1135,6 +1145,8 @@ match any font." );
   catalogcom[57]=dupstring( "Change layout" );
   catalogcom[58]=dupstring( "Volume manager" );
   catalogcom[59]=dupstring( "Switch button bank" );
+  catalogcom[60]=dupstring( "Path Jump" );
+  catalogcom[61]=dupstring( "Copy string to clipboard" );
   
   catalogflag[0]=dupstring( "String requester" );
   catalogflag[1]=dupstring( "path" );
